# Create the order entry database's tables.
# Be sure to create the tables in the right order 
# so you can create their referential constraints.

# Create the database.
# Comment this statement out if the database already exists
# or if you are using an Access database.
CREATE DATABASE OrderEntry
ON
  (NAME = oe_data,
   FILENAME = 'C:\Temp\OrderEntry.mdf',
   SIZE = 2MB,
   MAXSIZE = 10MB,
   FILEGROWTH = 2MB
)
LOG ON
  (NAME = oe_log,
   FILENAME = 'C:\Temp\OrderEntry.ldf',
   SIZE = 1MB,
   MAXSIZE = 5MB,
   FILEGROWTH = 1MB
);

# Use the new database.
# Comment this statement out if you are using an Access database.
USE OrderEntry;

################
# Range Tables #
################
# Range table for Zip codes in our operating area.
# The Zip code must have format 12345 or 12345-6789.
CREATE TABLE Zips (
    Zip VARCHAR(10) UNIQUE NOT NULL
        CHECK ((Zip LIKE '[0-9][0-9][0-9][0-9][0-9]') OR
               (Zip LIKE '[0-9][0-9][0-9][0-9][0-9]-[0-9][0-9][0-9][0-9]'))
);

# Range table for states in our operating area.
CREATE TABLE States (
    Abbreviation CHAR(2)     UNIQUE NOT NULL,
    Name         VARCHAR(20) UNIQUE NOT NULL
);

# Range table for cities in our operating area.
CREATE TABLE Cities (
    Name         VARCHAR(40) UNIQUE NOT NULL
);

# Range table for address types.
CREATE TABLE AddressTypes (
    AddressType  VARCHAR(20) UNIQUE NOT NULL
);

# Range table for date types.
CREATE TABLE DateTypes (
    DateType     VARCHAR(20) UNIQUE NOT NULL
);

####################
# Main Data Tables #
####################
# Inventory.
CREATE TABLE Inventory (
    ItemNumber      INT           IDENTITY(1,1) PRIMARY KEY,
    QuantityInStock INT           NOT NULL,
    Description     VARCHAR(40)   NOT NULL,
    UnitPrice       SMALLMONEY    NOT NULL
);

# Users.
CREATE TABLE Users (
    UserId          VARCHAR(12)   UNIQUE NOT NULL,
    FirstName       VARCHAR(40)   NOT NULL,
    LastName        VARCHAR(40)   NOT NULL
);

# Customers.
CREATE TABLE Customers (
    CustomerId      INT           IDENTITY(1,1) PRIMARY KEY,
    CustomerName    VARCHAR(40)   UNIQUE NOT NULL
);

# CustomerAddresses.
CREATE TABLE CustomerAddresses (
    CustomerId      INT           REFERENCES Customers (CustomerId),
    AddressType     VARCHAR(20)   REFERENCES AddressTypes (AddressType),
    Street          VARCHAR(40)   NOT NULL,
    City            VARCHAR(40)   REFERENCES Cities (Name),
    State           CHAR(2)       REFERENCES States (Abbreviation),
    Zip             VARCHAR(10)   REFERENCES Zips (Zip)
);

# CustomerOrders.
CREATE TABLE CustomerOrders (
    OrderId         INT           IDENTITY(1, 1) PRIMARY KEY,
    CustomerId      INT           REFERENCES Customers (CustomerId),
    ContactName     VARCHAR(80),
    InvoiceNumber   VARCHAR(10),
    EnteredByUser   VARCHAR(12)   REFERENCES Users (UserId)
);

# The following tables reference the CustomerOrders table.
# They are essentially detail for CustomerOrders.
# OrderItems.
CREATE TABLE OrderItems (
    OrderId         INT           REFERENCES CustomerOrders (OrderId),
    ItemNumber      INT           REFERENCES Inventory (ItemNumber),
    QuantityOrdered INT           NOT NULL
);

# OrderDates.
CREATE TABLE OrderDates (
    OrderId         INT           REFERENCES CustomerOrders (OrderId),
    DateType        VARCHAR(20)   REFERENCES DateTypes (DateType),
    DateValue       SMALLDATETIME NOT NULL
);
